﻿using anbs_cp.Interfaces;

namespace anbs_cp.Classes;

/// <summary>
/// Абстрактный класс конвертера величин для отображения (улучшенный аналог ValueFormatter)
/// </summary>
public abstract class ValueConverter: IValueConverter
{
    /// <summary>
    /// Конструктор
    /// </summary>
    /// <param name="valueNames">Массив имён размерностей</param>
    /// <param name="divider">Делитель</param>
    protected ValueConverter (string[] valueNames, long divider)
    {
        ValueNames = valueNames;
        Divider = divider;
    }

    #region Реализация интерфейса
    /// <summary>
    /// Массив имён размерностей
    /// </summary>
    public string[] ValueNames { get; init; }

    /// <summary>
    /// Делитель
    /// </summary>
    public long Divider { get; init; }
    #endregion

    #region Методы
    /// <summary>
    /// Функция конвертирования в строку
    /// </summary>
    /// <param name="value">Значение</param>
    /// <returns>Конвертирование значение в строку</returns>
    public string Convert (long value)
    {
        (decimal, int) result = DivideIt(value, 0);

        return $"{result.Item1:F2} {ValueNames[result.Item2]}";
    }


    /// <summary>
    /// Рекурсивная функция деления
    /// </summary>
    /// <param name="value">Число</param>
    /// <param name="count">Счётчик вызова рекурсии</param>
    /// <returns>Число в остатке и количество вызовов рекурсии</returns>
    private (decimal, int) DivideIt (decimal value, int count)
    {
        //Если счёт уже больше количества названий
        if (count > ValueNames.Length)
            return (value, count);

        //Если частное уже меньше делителя, то прерываем цикл
        if (value < Divider)
            return (value, count);

        //Увеличиваем счётчик...
        count++;

        //... и продолжаем цикл
        return  DivideIt(value / Divider, count);


    }
    #endregion
}